#!/bin/bash

AWK="/usr/bin/awk"
COMM="/usr/bin/comm"
ECHO="/bin/echo"
EGREP="/bin/egrep"
GREP="/usr/bin/grep"
ID="/usr/xpg4/bin/id"
INETADM="/usr/sbin/inetadm"
LS="/bin/ls"
PMADM="/usr/sbin/pmadm"
SED="/usr/bin/sed"
SORT="/usr/bin/sort"
SVCADM="/usr/sbin/svcadm"
SVCCFG="/usr/sbin/svccfg"
SVCS="/usr/bin/svcs"
TR="/bin/tr"
UNAME="/bin/uname"
UNIQ="/usr/bin/uniq"
WC="/bin/wc"

OS="unknown"
OS_VERSION="unknown"

SUCCESS=0
ERROR=1

function print_error {
   ${ECHO} "Error: $@" >&2
}

function print_warn {
   ${ECHO} "WARNING: $@"
}

function print_info {
   ${ECHO} "INFO: $@"
}

function print_ok {
   ${ECHO} "OK: $@"
}

function print_banner {
   ${ECHO} "[ $@ ]"
}

function print_divider {
   ${ECHO} "==========================================================================================="
}

function check_os {
   OPERATING_SYSTEM=$( ${UNAME} -s )   
   OS_REVISION=$( ${UNAME} -r )
   case "${OPERATING_SYSTEM}" in
      "SunOS")	OS="Solaris"
		case "${OS_REVISION}" in
		   "5.10")  OS_VERSION="10" ;;
		   "5.9" )  OS_VERSION="9"  ;;
		   "5.8" )  OS_VERSION="8"  ;;
                   "5.7" )  OS_VERSION="7"  ;;
                   *     )  OS_VERSION="UNSUPPORTED" ;;
                esac ;;
       *     )  OS="UNSUPPORTED" ;;
   esac
   if [ "${OS}" = "UNSUPPORTED" -o "${OS_VERSION}" = "UNSUPPORTED" ]; then
      print_error "Sorry, ${OPERATING_SYSTEM} ${OS_REVISION} is not supported"
      exit ${ERROR}
   elif [ "${OS_VERSION}" -ne "10" ]; then   # temporary....
      print_error "Sorry, only Solaris 10 supported at present"
      exit ${ERROR}
   fi
}

function check_user {
   MY_UID=$( ${ID} -u )
   if [ "${MY_UID}" -ne "0" ]; then
      print_error "This script must be executed as root"
      exit ${ERROR}
   fi
}

function check_login_serial {
   print_banner "Checking for \"login:\" Prompts on Serial Ports"
   print_divider
   case ${OS_VERSION} in
      "10")
          ${PMADM} -l | ${SED} '1d' | ${AWK} '{print $3, $4}' | while read S_TTY S_FLAGS; do
             ${ECHO} "${S_FLAGS}" | ${GREP} '^.*x.*$' >/dev/null 2>&1
             if [ "$?" -ne "0" ]; then
                print_warn "login: prompt enabled on serial port ${S_TTY}"
                print_info "To disable, execute the following command:"
                print_info "pmadm -d -p zsmon -s ${S_TTY}"
             else
                print_ok "login: prompt disabled on serial port ${S_TTY}"
             fi 
          done
          ;;
   esac
}

function check_nobody_keys {
   print_banner "Checking \"nobody\" Access for Secure RPC"
   print_divider
   case ${OS_VERSION} in
     "10")
         ${GREP} "^ENABLE_NOBODY_KEYS=NO" /etc/default/keyserv
         if [ "$?" -ne "0" ]; then
            print_warn "\"nobody\" access to Secure RPC not explicitly disabled in /etc/default/keyserv"
            print_info "Execute the following commands to disable:"
            print_info "cd /etc/default"
            print_info "sed 's/^[#]\{0,1\}ENABLE_NOBODY_KEYS=.*$/ENABLE_NOBODY_KEYS=NO/' keyserv > keyserv.new"
            print_info "mv keyserv.new keyserv"
            print_info "pkgchk -f -n -p /etc/default/keyserv"
         else
            print_ok "Access for \"nobody\" to Secure RPC disabled"
         fi
         ;;
   esac
}

function check_sshd_config {
   print_banner "Checking SSH Daemon Configuration"
   print_divider
   case ${OS_VERSION} in 
     "10")
         print_info "Where warnings are found whilst parsing /etc/ssh/sshd_config, either"
         print_info "uncomment and/or fix the entries contained therein"
         ${GREP} "^Protocol[ ][ ]*2$" /etc/ssh/sshd_config >/dev/null 2>&1
         if [ "$?" -ne "0" ]; then
            print_warn "\"Protocol\" not set to \"2\" in /etc/ssh/sshd_config"
         else
            print_ok "\"Protocol\" set to \"2\" in /etc/ssh/sshd_config"
         fi
         ${GREP} "^X11Forwarding[ ][ ]*yes$" /etc/ssh/sshd_config >/dev/null 2>&1
         if [ "$?" -ne "0" ]; then
            print_warn "\"X11Forwarding\" not set to \"yes\" in /etc/ssh/sshd_config"
         else
            print_ok "\"X11Forwarding\" set to \"yes\" in /etc/ssh/sshd_config"
         fi
         ${GREP} "^MaxAuthTries[ ][ ]*5$" /etc/ssh/sshd_config >/dev/null 2>&1
         if [ "$?" -ne "0" ]; then
            print_warn "\"MaxAuthTries\" not set to \"5\" in /etc/ssh/sshd_config"
         else
            print_ok "\"MaxAuthTries\" set to \"5\" in /etc/ssh/sshd_config"
         fi
         ${GREP} "^MaxAuthTriesLog[ ][ ]*0$" /etc/ssh/sshd_config >/dev/null 2>&1
         if [ "$?" -ne "0" ]; then
            print_warn "\"MaxAuthTriesLog\" not set to \"0\" in /etc/ssh/sshd_config"
         else
            print_ok "\"MaxAuthTriesLog\" set to \"0\" in /etc/ssh/sshd_config"
         fi
         ${GREP} "^RhostsAuthentication[ ][ ]*no$" /etc/ssh/sshd_config >/dev/null 2>&1
         if [ "$?" -ne "0" ]; then
            print_warn "\"RhostsAuthentication\" not set to \"no\" in /etc/ssh/sshd_config"
         else
            print_ok "\"RhostsAuthentication\" set to \"no\" in /etc/ssh/sshd_config"
         fi
         ${GREP} "^RhostsRSAAuthentication[ ][ ]*no$" /etc/ssh/sshd_config >/dev/null 2>&1
         if [ "$?" -ne "0" ]; then
            print_warn "\"RhostsRSAAuthentication\" not set to \"no\" in /etc/ssh/sshd_config"
         else
            print_ok "\"RhostsRSAAuthentication\" set to \"no\" in /etc/ssh/sshd_config"
         fi
         ${GREP} "^PermitRootLogin[ ][ ]*no$" /etc/ssh/sshd_config >/dev/null 2>&1
         if [ "$?" -ne "0" ]; then
            print_warn "\"PermitRootLogin\" not set to \"no\" in /etc/ssh/sshd_config"
         else
            print_ok "\"PermitRootLogin\" set to \"no\" in /etc/ssh/sshd_config"
         fi
         ${GREP} "^PermitEmptyPasswords[ ][ ]*no$" /etc/ssh/sshd_config >/dev/null 2>&1
         if [ "$?" -ne "0" ]; then
            print_warn "\"PermitEmptyPasswords\" not set to \"no\" in /etc/ssh/sshd_config"
         else
            print_ok "\"PermitEmptyPasswords\" set to \"no\" in /etc/ssh/sshd_config"
         fi
         ${GREP} "^Banner[ ][ ]*/etc/issue$" /etc/ssh/sshd_config >/dev/null 2>&1
         if [ "$?" -ne "0" ]; then
            print_warn "\"Banner\" not set to \"/etc/issue\" in /etc/ssh/sshd_config"
         else
            print_ok "\"Banner\" set to \"/etc/issue\" in /etc/ssh/sshd_config"
         fi
         ;;
   esac
}

function check_pam_rhosts_support {
   print_banner "Checking for .rhosts Support in /etc/pam.conf"
   print_divider
   case ${OS_VERSION} in 
     "10")
         ${GREP} '^[^#].*pam_rhosts_auth.so.1$' /etc/pam.conf >/dev/null 2>&1
         if [ "$?" -eq "0" ]; then
            print_warn ".rhosts support exists in /etc/pam.conf"
            print_info "It is recommended that you remove .rhosts support in /etc/pam.conf"
            print_info "Execute the following commands to do this:"
            print_info "cd /etc"
            print_info "grep -v \"pam_rhosts_auth\.so\.1\" pam.conf > pam.conf.new"
            print_info "mv pam.conf.new pam.conf"
            print_info "pkgchk -f -n -p /etc/pam.conf"
         else
            print_ok ".rhosts support does not exist in /etc/pam.conf"
         fi 
         ;;
   esac
}

function check_ftpusers {
   print_banner "Checking /etc/ftpd/ftpusers"
   print_divider
   case ${OS_VERSION} in
     "10")
         ${SVCS} svc:/network/ftp:default >/dev/null 2>&1
         if [ "$?" -eq "0" ]; then
            FTP_STATUS=$( ${SVCS} svc:/network/ftp:default | ${SED} -n '$p' | ${AWK} '{print $1}' )
            if [ "${FTP_STATUS}" != "disabled" ]; then
               if [ ! -e "/etc/ftpd/ftpusers" ]; then
                  print_warn "File /etc/ftpd/ftpusers does not exist"
               else
                  print_info "Checking /etc/ftpd/ftpusers for the following users:"
                  USERLIST="root|daemon|bin|sys|adm|lp|uucp|nuucp|smmsp|listen"
                  USERLIST="${USERLIST}|gdm|webserverd|nobody|noaccess|nobody4"
                  USERCOUNT=$( ${ECHO} "${USERLIST}" | ${TR} "|" "\n" | ${WC} -l )
                  ${ECHO} "${USERLIST}" | ${TR} "|" "\n" | while read USERNAME; do
                     print_info "${USERNAME}"
                  done
                  USERCHECK=$( eval ${EGREP} '^\(${USERLIST}\)$' /etc/ftpd/ftpusers )
                  USERCHECKCOUNT=$( ${ECHO} "${USERCHECK}" | ${SORT} | ${UNIQ} | ${WC} -l )
                  if [ "${USERCHECKCOUNT}" -ne "${USERCOUNT}" ]; then
                     print_warn "Not all accounts listed are present in /etc/ftpd/ftpusers"
                     ${COMM} -23 <( ${ECHO} "${USERLIST}" | ${TR} "|" "\n" | ${SORT} ) <( ${ECHO} "${USERCHECK}" | "${SORT}" | "${UNIQ}" ) |\
                        while read MISSINGACCOUNT; do
                           print_info "Account \"${MISSINGACCOUNT}\" should be added to /etc/ftpd/ftpusers"
                        done
                  fi 
               fi
                              
            else 
               print_ok "Service \"svc:/network/ftp:default\" is disabled"
            fi
         else
            print_ok "Service \"svc:/network/ftp:default\" does not exist"
         fi
         ;;
   esac
}

function check_sendmail_listen {
   print_banner "Checking That sendmail Only Listens Locally"
   print_divider
   case ${OS_VERSION} in
     "10")
         print_info "Checking is sendmail is disabled from listening on external interfaces"
         print_info "Obviously if this is a mail server, ignore these test results"
         ${GREP} "^O DaemonPortOptions=.*inet.*$" /etc/mail/sendmail.cf | ${GREP} -v "inet6" >/dev/null 2>&1
         if [ "$?" -ne "0" ]; then
            print_warn "DaemonPortOptions not set for IPv4 in /etc/mail/sendmail.cf"
         else
            ${GREP} "^O DaemonPortOptions=.*inet.*$" /etc/mail/sendmail.cf | ${GREP} -v "inet6" |\
               ${GREP} "Addr=127\.0\.0\.1" >/dev/null 2>&1
            if [ "$?" -ne "0" ]; then
               print_warn "sendmail not configured to listen locally only on port 25"
               print_info "To configure, execute the following commands:"
               print_info "sed 's/^\(O DaemonPortOptions.*inet\)$/\1, Addr=127.0.0.1/' /etc/mail/sendmail.cf > /etc/mail/sendmail.cf.new"
               print_info "mv /etc/mail/sendmail.cf.new /etc/mail/sendmail.cf"
               print_info "svcadm refresh svc:/network/smtp:sendmail"
            else
               print_ok "sendmail configured to listen locally only on port 25"
            fi
         fi
         ${GREP} "^O DaemonPortOptions.*inet6.*$" /etc/mail/sendmail.cf >/dev/null 2>&1
         if [ "$?" -eq "0" ]; then # don't care if it's already commented out or not there
            print_warn "sendmail configured to listen on IPv6 addresses"  
            print_info "Unless you use IPv6 at your site, this isn't required"
            print_info "To disable sendmail on IPv6 addresses, enter the following commands:"
            print_info "sed 's/^\(O DaemonPortOptions.*inet6.*\)$/#\1/' /etc/mail/sendmail.cf > /etc/mail/sendmail.cf.new"
            print_info "mv /etc/mail/sendmail.cf.new /etc/mail/sendmail.cf"
            print_info "svcadm refresh svc:/network/smtp:sendmail"
         fi 
         ;;
   esac
}

function check_syslog_remote {
   print_banner "Checking That syslogd Will Not Accept Remote Messages"
   print_divider
   case ${OS_VERSION} in
     "10")
         ${GREP} "^LOG_FROM_REMOTE=NO" /etc/default/syslogd >/dev/null 2>&1
         if [ "$?" -ne "0" ]; then
            print_warn "syslogd not configured to reject remote messages"
            print_info "It is recommended that you do not allow syslogd to listen for"
            print_info "messages from other systems, unless this server is a log server"
            print_info "To disable, execute the following commands:"
            print_info "cd /etc/default" 
            print_info "sed 's/^[#]\{0,1\}LOG_FROM_REMOTE=.*$/LOG_FROM_REMOTE=NO/' syslogd > syslogd.new"
            print_info "mv syslogd.new syslogd"
            print_info "svcadm refresh svc:/system/system-log:default"
         else
            print_ok "syslogd correctly configured"
         fi
         ;;
   esac
}

function check_tcp_wrappers {
   # Only a basic check, doesn't check contents of files
   print_banner "Checking That TCP Wrappers is Enabled"
   print_divider
   case ${OS_VERSION} in
     "10")
         TCP_WRAPPERS_FILES="/etc/hosts.allow /etc/hosts.deny"
         for TCP_WRAPPERS_FILE in ${TCP_WRAPPERS_FILES}; do
            if [ ! -e "${TCP_WRAPPERS_FILE}" ]; then
               print_warn "${TCP_WRAPPERS_FILE} does not exist"
            else
               print_ok "${TCP_WRAPPERS_FILE} exits" 
            fi
         done
         TCP_WRAPPERS_STATE=$( ${INETADM} -p | ${GREP} "^tcp_wrappers" | ${SED} 's/^tcp_wrappers=\(.*\)$/\1/' )
         if [ "${TCP_WRAPPERS_STATE}" != "TRUE" ]; then         
            print_warn "Global TCP Wrappers configuration not enabled"
            print_info "To enable, execute the following command:"
            print_info "inetadm -M tcp_wrappers=TRUE" 
         else
            print_ok "Global TCP Wrappers configuration enabled"
         fi
         ;;
   esac   
}

function check_crontabs {
   print_banner "Checking crontabs"
   print_divider
   case ${OS_VERSION} in
     "10")
         CRON_ERR_COUNT=0
         for CRONTAB in /var/spool/cron/crontabs/*; do
            LINECOUNT=$( ${GREP} -v "^#" ${CRONTAB} | ${GREP} -v "^[ 	]*$" | ${WC} -l )
            if [ "${LINECOUNT}" -eq "0" ]; then
               print_info "The crontab [${CRONTAB}] is empty - consider removing it"
               print_info "with \"crontab -r ${CRONTAB##*/}\""
            fi
            CRONTAB_OWNER=$( ${LS} -ld ${CRONTAB} | ${AWK} '{print $3}' )
            CRONTAB_GROUP=$( ${LS} -ld ${CRONTAB} | ${AWK} '{print $4}' )
            CRONTAB_PERMS=$( ${LS} -ld ${CRONTAB} | ${AWK} '{print $1}' )
            if [ "${CRONTAB_OWNER}" != "root" ]; then
               print_warn "${CRONTAB} is not owned by root. Execute the following command to fix it:"
               print_info "chown root ${CRONTAB}"
               (( CRON_ERR_COUNT = CRON_ERR_COUNT + 1 ))
            fi
            if [ "${CRONTAB_GROUP}" != "root" ]; then
               print_warn "${CRONTAB} is not owned by group sys. Execute the following command to fix it:"
               print_info "chgrp sys ${CRONTAB}"
               (( CRON_ERR_COUNT = CRON_ERR_COUNT + 1 ))
            fi
            if [ "${CRONTAB_PERMS}" != "-r--------" ]; then
               print_warn "${CRONTAB} is not mode 400. Execute the following command to fix it:"
               print_info "chmod 400 ${CRONTAB}"
               (( CRON_ERR_COUNT = CRON_ERR_COUNT + 1 ))
            fi
         done
         if [ "${CRON_ERR_COUNT}" -eq "0" ]; then
            print_ok "All crontab permissions and ownership OK"
         fi
         ;;
   esac
}

function check_cron_access {
   print_banner "Checking cron Facility Access"
   print_divider
   # We just check files exist, and that "allow" files contain "root"
   case ${OS_VERSION} in
     "10")
         NO_FILES="/etc/cron.d/cron.deny /etc/cron.d/at.deny"
         YES_FILES="/etc/cron.d/cron.allow /etc/cron.d/at.allow"
         for YES_FILE in ${YES_FILES}; do
            if [ ! -e "${YES_FILE}" ]; then
               print_warn "${YES_FILE} does not exist"
            else
               ${GREP} "^root$" ${YES_FILE} >/dev/null 2>&1
               if [ "$?" -ne "0" ]; then
                  print_warn "root not allowed to use $( ${ECHO} ${YES_FILE} | ${SED} 's/\..*//' )"
               else
                  print_ok "root allowed to use $( ${ECHO} ${YES_FILE} | ${SED} 's/\..*//' )"
               fi
            fi
         done
         for NO_FILE in ${NO_FILES}; do
            if [ -e "${NO_FILE}" ]; then
               print_warn "No need for ${NO_FILE} ( if ${NO_FILE/deny/allow} exists )" 
            else
               print_ok "${NO_FILE} does not exist - this is good if ${NO_FILE/deny/allow} exists"
            fi
         done
         ;;
   esac
}

function check_console_login {
   print_banner "Checking That root Can Only Log in on /dev/console"
   print_divider
   case ${OS_VERSION} in
     "10")
         ${GREP} "^CONSOLE=/dev/console" /etc/default/login >/dev/null 2>&1
         if [ "$?" -ne "0" ]; then
            print_warn "CONSOLE variable not correctly configured in /etc/default/login"
            print_info "To disable non-/dev/console root logins, execute the following commands:"
            print_info "cd /etc/default" 
            print_info "sed 's!^[#]\{0,1\}CONSOLE=.*\$!CONSOLE=/dev/console!' login > login.new"
            print_info "mv login.new login"
            print_info "pkgchk -f -n -p /etc/default/login"
         else
            print_ok "CONSOLE variable correctly configured in /etc/default/login"
         fi
         ;;
   esac
}

function check_account_lockout {
   print_banner "Checking Account Lockout Policy"
   print_divider
   case ${OS_VERSION} in
     "10")
         ${GREP} "^RETRIES=[1-5]$" /etc/default/login >/dev/null 2>&1
         if [ "$?" -ne "0" ]; then
            print_warn "Bad login attempt retry limit not set to 5 (or less) in /etc/default/login"
            print_info "To force reconnection after 5 bad login attempts, execute the following commands:"
            print_info "cd /etc/default" 
            print_info "sed 's/^[#]\{0,1\}RETRIES=.*$/RETRIES=5/' login > login.new"
            print_info "mv login.new login"
            print_info "pkgchk -f -n -p /etc/default/login"
         else
            print_ok "Bad login attempt retry limit set correctly"
         fi
         ${GREP} "^LOCK_AFTER_RETRIES=YES" /etc/security/policy.conf >/dev/null 2>&1
         if [ "$?" -ne "0" ]; then
            print_warn "Accounts not set to lock after RETRIES limit reached"
            print_info "To lock accounts after RETRIES limit reached, execute the following commands:"
            print_info "cd /etc/security" 
            print_info "sed 's/^[#]\{0,1\}LOCK_AFTER_RETRIES=.*$/LOCK_AFTER_RETRIES=YES/' policy.conf > policy.conf.new"
            print_info "mv policy.conf.new policy.conf"
            print_info "pkgchk -f -n -p /etc/security/policy.conf"
            print_info "You can then disable lockout for particular users with:"
            print_info "usermod -K lock_after_retries=no username"
         else
            print_ok "Account lockout policy defined correctly"
         fi
         ;;
   esac
}

check_os
check_user
print_divider
check_login_serial
print_divider
check_nobody_keys
print_divider
check_sshd_config
print_divider
check_pam_rhosts_support
print_divider
check_ftpusers
print_divider
check_sendmail_listen
print_divider
check_syslog_remote
print_divider
# we don't check X config, as X should be disabled 
check_tcp_wrappers
print_divider
check_crontabs
print_divider
check_cron_access
print_divider
check_console_login
print_divider
check_account_lockout
print_divider
# don't check eeprom security - we have physical security

exit 0
